// @bun
// src/assistant/index.ts
import { ViewPlugin as ViewPlugin2 } from "@codemirror/view";

// src/assistant/lib/state.ts
import { StateEffect, StateField } from "@codemirror/state";
var switchTabEffect = StateEffect.define();
var addMessageEffect = StateEffect.define();
var updateMessageStatusEffect = StateEffect.define();
var selectModelEffect = StateEffect.define();
var setApiKeyEffect = StateEffect.define();
var toggleSettingsEffect = StateEffect.define();
var assistantState = StateField.define({
  create() {
    const defaultModelId = "gpt-4";
    const savedApiKeys = localStorage.getItem("cm-ai-api-keys");
    const apiKeys = savedApiKeys ? JSON.parse(savedApiKeys) : {
      google: "",
      openai: "",
      mistral: ""
    };
    return {
      activeTab: "assistant",
      messages: [],
      isLoading: false,
      selectedModel: defaultModelId,
      apiKeys,
      showSettings: false
    };
  },
  update(value, tr) {
    for (const effect of tr.effects) {
      if (effect.is(switchTabEffect)) {
        return { ...value, activeTab: effect.value };
      } else if (effect.is(addMessageEffect)) {
        return {
          ...value,
          messages: [...value.messages, effect.value],
          isLoading: effect.value.status === "sending" || effect.value.status === "streaming"
        };
      } else if (effect.is(updateMessageStatusEffect)) {
        return {
          ...value,
          messages: value.messages.map((msg) => msg.id === effect.value.message.id ? {
            ...msg,
            status: effect.value.status,
            content: effect.value.content ?? msg.content
          } : msg),
          isLoading: effect.value.status === "sending" || effect.value.status === "streaming"
        };
      } else if (effect.is(selectModelEffect)) {
        return { ...value, selectedModel: effect.value };
      } else if (effect.is(setApiKeyEffect)) {
        const newState = {
          ...value,
          apiKeys: {
            ...value.apiKeys,
            [effect.value.provider]: effect.value.key
          }
        };
        localStorage.setItem("cm-ai-api-keys", JSON.stringify(newState.apiKeys));
        return newState;
      } else if (effect.is(toggleSettingsEffect)) {
        return { ...value, showSettings: effect.value };
      }
    }
    return value;
  }
});

// src/assistant/lib/renderer.ts
import crelt from "crelt";

// src/assistant/lib/codeblock.ts
import { StateField as StateField2, StateEffect as StateEffect2, EditorState } from "@codemirror/state";
import { EditorView, Decoration, WidgetType } from "@codemirror/view";
import { javascript } from "@codemirror/lang-javascript";
import { python } from "@codemirror/lang-python";
import { markdown } from "@codemirror/lang-markdown";
import { oneDark } from "@codemirror/theme-one-dark";
var getLanguageSupport = (lang) => {
  const normalizedLang = lang.toLowerCase().trim();
  switch (normalizedLang) {
    case "javascript":
    case "js":
      return javascript();
    case "typescript":
    case "ts":
      return javascript({ typescript: true });
    case "python":
    case "py":
      return python();
    case "markdown":
    case "md":
      return markdown();
    default:
      return null;
  }
};

class CodeBlockWidget extends WidgetType {
  code;
  language;
  isIncomplete;
  view = null;
  static chunkCounter = 0;
  constructor(code, language, isIncomplete = false) {
    super();
    this.code = code;
    this.language = language;
    this.isIncomplete = isIncomplete;
  }
  eq(other) {
    return this.code === other.code && this.language === other.language && this.isIncomplete === other.isIncomplete;
  }
  destroy() {
    this.view?.destroy();
  }
  toDOM() {
    const wrapper = document.createElement("div");
    wrapper.className = "cm-ext-assistant-code";
    if (this.language) {
      const header = document.createElement("div");
      header.className = "cm-ext-assistant-code-header";
      const dot = document.createElement("span");
      dot.textContent = "\u25CF";
      dot.style.marginRight = "6px";
      header.appendChild(dot);
      const langText = document.createElement("span");
      langText.textContent = this.language;
      header.appendChild(langText);
      if (this.isIncomplete) {
        CodeBlockWidget.chunkCounter = (CodeBlockWidget.chunkCounter + 1) % 3;
        const loadingDots = document.createElement("div");
        loadingDots.className = "cm-ext-assistant-loading-dots";
        for (let i = 0;i < 3; i++) {
          const dot2 = document.createElement("div");
          dot2.className = `cm-ext-assistant-loading-dot ${i <= CodeBlockWidget.chunkCounter ? "filled" : "empty"}`;
          loadingDots.appendChild(dot2);
        }
        header.appendChild(loadingDots);
      }
      if (!this.isIncomplete) {
        const copyButton = document.createElement("button");
        copyButton.className = "cm-ext-assistant-code-copy";
        copyButton.title = "Copy code";
        copyButton.innerHTML = `<svg width="14" height="14" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round">
                    <rect x="9" y="9" width="13" height="13" rx="2" ry="2"></rect>
                    <path d="M5 15H4a2 2 0 0 1-2-2V4a2 2 0 0 1 2-2h9a2 2 0 0 1 2 2v1"></path>
                </svg>`;
        copyButton.addEventListener("click", async () => {
          try {
            await navigator.clipboard.writeText(this.code);
            const originalHTML = copyButton.innerHTML;
            copyButton.innerHTML = `<svg width="14" height="14" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round">
                            <path d="M20 6L9 17l-5-5"></path>
                        </svg>`;
            setTimeout(() => {
              copyButton.innerHTML = originalHTML;
            }, 2000);
          } catch (err) {
            console.error("Failed to copy code:", err);
          }
        });
        header.appendChild(copyButton);
      }
      wrapper.appendChild(header);
    }
    if (!this.isIncomplete) {
      const editorContainer = document.createElement("div");
      editorContainer.className = "cm-ext-assistant-code-editor";
      const languageSupport = this.language ? getLanguageSupport(this.language) : null;
      const extensions = [
        EditorView.editable.of(false),
        EditorView.lineWrapping,
        EditorState.readOnly.of(true),
        oneDark
      ];
      if (languageSupport) {
        extensions.push(languageSupport);
      }
      this.view = new EditorView({
        state: EditorState.create({
          doc: this.code,
          extensions
        }),
        parent: editorContainer
      });
      wrapper.appendChild(editorContainer);
    }
    return wrapper;
  }
}
var addCodeBlock = StateEffect2.define();
var removeCodeBlocks = StateEffect2.define();
var codeBlockState = StateField2.define({
  create() {
    return Decoration.none;
  },
  update(decorations, tr) {
    decorations = decorations.map(tr.changes);
    for (const effect of tr.effects) {
      if (effect.is(addCodeBlock)) {
        const { from, to, code, language } = effect.value;
        const widget = new CodeBlockWidget(code, language);
        const deco = Decoration.replace({
          widget,
          inclusive: true,
          block: true
        });
        decorations = decorations.update({
          add: [{ from, to, value: deco }],
          filter: (from2, to2) => from2 >= effect.value.to || to2 <= effect.value.from
        });
      } else if (effect.is(removeCodeBlocks)) {
        decorations = Decoration.none;
      }
    }
    return decorations;
  },
  provide: (f) => EditorView.decorations.from(f)
});
var parseCodeBlocks = (text) => {
  const codeBlocks = [];
  const codeBlockRegex = /```([a-zA-Z]*)\n([\s\S]*?)\n```/g;
  let match;
  while ((match = codeBlockRegex.exec(text)) !== null) {
    codeBlocks.push({
      from: match.index,
      to: match.index + match[0].length,
      language: match[1] || null,
      code: match[2]
    });
  }
  return codeBlocks;
};

// src/ai/ai.ts
import { streamText } from "ai";
import { createOpenAI } from "@ai-sdk/openai";
import { createMistral } from "@ai-sdk/mistral";
import { createGoogleGenerativeAI } from "@ai-sdk/google";

// src/utils/logger.ts
import pino from "pino";
var isProduction = false;
var logger = pino({
  level: "debug",
  transport: !isProduction ? {
    target: "pino-pretty",
    options: {
      colorize: true,
      translateTime: "SYS:standard"
    }
  } : undefined
});
var logger_default = logger;

// src/ai/ai.ts
var DEFAULT_MODEL = "openai:gpt-4o";
var debug = (...args) => logger_default.debug({ module: "AI" }, args.join(" "));
var createAIService = () => {
  const generateTextFn = async ({
    modelName,
    prompt,
    editorContent,
    apiKey,
    onTextContent
  }) => {
    if (!apiKey) {
      throw new Error("API key is required. Please configure it in settings.");
    }
    const fullPrompt = `Current Editor Content:
${editorContent}

${prompt}`;
    let result;
    let fullText = "";
    debug("Generating text for model:", modelName);
    debug("Prompt:", fullPrompt);
    try {
      const systemPrompt = "You are a helpful assistant that can help with coding tasks.";
      switch (modelName) {
        case "openai:gpt-4o": {
          const simulatedResponse = [
            `# Here's a simulated response with code examples

`,
            "## Let's start with a simple inline code example: ",
            "`console.log('hello world')`\n\n",
            "Here's a multiline code block:\n\n```jav",
            `ascript
function greet(name) {
`,
            "    console.log(`Hello, ${name}!`)\n    return {\n}\n",
            "    console.log(`Hello, ${name}!`)\n    return {\n",
            `        message: 'Greeting sent',
        timestamp: new Date()
    }
}

`,
            `// Example usage
const result = greet('Developer')
\`\`\`

`,
            "You can also use inline code for variables like `result` or ",
            `\`name\`.

Here's another code block with a different language:

`,
            "```python\ndef calculate_sum(numbers):\n    total = 0\n",
            `    for num in numbers:
        total += num
    return total

`,
            '# Test the function\nnumbers = [1, 2, 3, 4, 5]\nprint(f"Sum: {calculate_sum(numbers)}")\n```'
          ];
          result = {
            textStream: async function* () {
              for (const chunk of simulatedResponse) {
                await new Promise((resolve) => setTimeout(resolve, 400));
                yield chunk;
              }
            }()
          };
          break;
        }
        case "mistral:large": {
          const mistralClient = createMistral({
            apiKey
          });
          result = streamText({
            model: mistralClient("mistral-large-latest"),
            system: systemPrompt,
            prompt: fullPrompt
          });
          break;
        }
        case "google:gemini-2.0-flash-001": {
          const googleClient = createGoogleGenerativeAI({
            apiKey
          });
          result = streamText({
            model: googleClient("gemini-2.0-flash-001"),
            system: systemPrompt,
            prompt: fullPrompt
          });
          break;
        }
        default: {
          console.warn(`Unknown model: ${modelName}. Using default model.`);
          const openaiClient = createOpenAI({
            apiKey,
            compatibility: "strict"
          });
          result = streamText({
            model: openaiClient("gpt-4"),
            system: systemPrompt,
            prompt: fullPrompt
          });
        }
      }
      for await (const textPart of result.textStream) {
        fullText += textPart;
        onTextContent?.(fullText);
      }
      return fullText;
    } catch (error) {
      debug("Error:", error);
      throw error;
    }
  };
  return {
    generateText: generateTextFn
  };
};
var aiService = createAIService();

// src/assistant/constants.ts
var AVAILABLE_MODELS = [
  {
    id: "gemini-pro",
    provider: "google",
    name: "Gemini Pro",
    description: "Google's most capable model for text generation"
  },
  {
    id: "gpt-4",
    provider: "openai",
    name: "GPT-4o",
    description: "OpenAI's most capable model"
  },
  {
    id: "mistral-large",
    provider: "mistral",
    name: "Mistral Large",
    description: "Mistral's largest open model"
  }
];

// src/assistant/lib/utils.ts
import { javascript as javascript2 } from "@codemirror/lang-javascript";
import { markdown as markdown2 } from "@codemirror/lang-markdown";
import { python as python2 } from "@codemirror/lang-python";
var mapModelToAIService = (modelId) => {
  switch (modelId) {
    case "gemini-pro":
      return "google:gemini-2.0-flash-001";
    case "gpt-4":
      return "openai:gpt-4o";
    case "mistral-large":
      return "mistral:large";
    default:
      return DEFAULT_MODEL;
  }
};
var formatProviderName = (provider) => {
  switch (provider) {
    case "openai":
      return "OpenAI";
    default:
      return provider.charAt(0).toUpperCase() + provider.slice(1);
  }
};

// src/assistant/lib/renderer.ts
var debug2 = (...args) => console.log("[Assistant Renderer]", ...args);
function renderAssistantPanel(dom, view) {
  const state = view.state.field(assistantState);
  if (!document.querySelector("#cm-spin-keyframes")) {
    const style = document.createElement("style");
    style.id = "cm-spin-keyframes";
    style.textContent = `
            @keyframes cm-spin {
                0% { transform: rotate(0deg); }
                100% { transform: rotate(360deg); }
            }
        `;
    document.head.appendChild(style);
  }
  if (state.showSettings) {
    renderSettingsPanel(dom, view);
    return;
  }
  dom.innerHTML = "";
  dom.className = "cm-ext-assistant-container";
  const tabsContainer = crelt("div", { class: "cm-ext-assistant-tabs" });
  const tabsGroup = crelt("div", { class: "cm-ext-assistant-tabs-group" });
  const assistantTab = crelt("button", {
    class: "cm-ext-assistant-tab cm-ext-assistant-tab-active"
  }, "Assistant");
  tabsGroup.appendChild(assistantTab);
  const controlsContainer = crelt("div", {
    class: "cm-ext-assistant-controls"
  });
  const modelSelect = crelt("select", {
    class: "cm-ext-assistant-model-select"
  });
  AVAILABLE_MODELS.forEach((model) => {
    const option = crelt("option", { value: model.id });
    option.textContent = model.name;
    option.style.direction = "ltr";
    option.style.textAlign = "right";
    if (model.id === state.selectedModel) {
      option.selected = true;
    }
    modelSelect.appendChild(option);
  });
  modelSelect.addEventListener("change", () => {
    view.dispatch({
      effects: selectModelEffect.of(modelSelect.value)
    });
  });
  const settingsButton = crelt("button", { class: "cm-ext-assistant-settings-button" }, "\u2699\uFE0F");
  settingsButton.addEventListener("click", () => {
    debug2("Settings button clicked, dispatching toggleSettingsEffect(true)");
    debug2("State before dispatch:", view.state.field(assistantState));
    view.dispatch({
      effects: toggleSettingsEffect.of(true)
    });
    debug2("State after dispatch:", view.state.field(assistantState));
    debug2("showSettings value:", view.state.field(assistantState).showSettings);
  });
  controlsContainer.appendChild(modelSelect);
  controlsContainer.appendChild(settingsButton);
  tabsContainer.appendChild(tabsGroup);
  tabsContainer.appendChild(controlsContainer);
  dom.appendChild(tabsContainer);
  const messagesContainer = crelt("div", {
    class: "cm-ext-assistant-messages"
  });
  const scrollToBottom = () => {
    requestAnimationFrame(() => {
      messagesContainer.scrollTop = messagesContainer.scrollHeight;
    });
  };
  state.messages.forEach((message) => {
    renderMessage(message, messagesContainer, scrollToBottom);
  });
  dom.appendChild(messagesContainer);
  const inputContainer = crelt("div", { class: "cm-ext-assistant-input" });
  const textarea = crelt("textarea", {
    placeholder: "Ask anything, @ to mention, \u2191 to select",
    rows: "3",
    onkeydown: async (e) => {
      if (e.key === "Enter" && !e.shiftKey) {
        e.preventDefault();
        const target = e.target;
        const content = target.value.trim();
        if (content) {
          debug2("Sending message:", content);
          const selectedModel = AVAILABLE_MODELS.find((m) => m.id === state.selectedModel);
          if (!selectedModel) {
            console.error("Selected model not found");
            return;
          }
          const apiKey = state.apiKeys[selectedModel.provider];
          if (!apiKey) {
            view.dispatch({
              effects: addMessageEffect.of({
                id: crypto.randomUUID(),
                role: "assistant",
                content: `Error: Please configure the API key for ${selectedModel.name} in settings.`,
                status: "complete"
              })
            });
            return;
          }
          target.value = "";
          const userMessage = {
            id: crypto.randomUUID(),
            role: "user",
            content,
            status: "complete"
          };
          view.dispatch({
            effects: addMessageEffect.of(userMessage)
          });
          const assistantMessage = {
            id: crypto.randomUUID(),
            role: "assistant",
            content: "",
            status: "streaming"
          };
          view.dispatch({
            effects: addMessageEffect.of(assistantMessage)
          });
          try {
            const editorContent = view.state.doc.toString();
            let streamedContent = "";
            const aiResponse = await aiService.generateText({
              modelName: mapModelToAIService(state.selectedModel),
              prompt: content,
              editorContent,
              apiKey,
              onTextContent: (text) => {
                streamedContent = text;
                view.dispatch({
                  effects: [
                    updateMessageStatusEffect.of({
                      message: assistantMessage,
                      status: "streaming",
                      content: text
                    })
                  ]
                });
                const messagesContainer2 = view.dom.querySelector(".cm-assistant-content")?.querySelector('div[style*="overflow-y: auto"]');
                if (messagesContainer2) {
                  messagesContainer2.scrollTop = messagesContainer2.scrollHeight;
                }
              }
            });
            view.dispatch({
              effects: [
                updateMessageStatusEffect.of({
                  message: assistantMessage,
                  status: "complete",
                  content: streamedContent || aiResponse
                })
              ]
            });
          } catch (error) {
            view.dispatch({
              effects: [
                updateMessageStatusEffect.of({
                  message: assistantMessage,
                  status: "complete",
                  content: `Error: ${error instanceof Error ? error.message : "An unknown error occurred"}`
                })
              ]
            });
          }
          requestIdleCallback(() => {
            const textareas = view.dom.getElementsByTagName("textarea");
            if (textareas.length > 0) {
              textareas[0].focus();
            }
          });
        }
      }
    }
  });
  inputContainer.appendChild(textarea);
  dom.appendChild(inputContainer);
}
function renderSettingsPanel(dom, view) {
  const state = view.state.field(assistantState);
  dom.innerHTML = "";
  dom.className = "cm-ext-assistant-settings";
  const headerContainer = crelt("div", { class: "cm-ext-assistant-tabs" });
  const header = crelt("div", { class: "cm-ext-assistant-tab cm-ext-assistant-tab-active" }, "API Key Settings");
  const backButton = crelt("button", { class: "cm-ext-assistant-settings-button" }, "\u2190");
  backButton.addEventListener("click", () => {
    view.dispatch({
      effects: toggleSettingsEffect.of(false)
    });
    requestIdleCallback(() => {
      const textareas = view.dom.getElementsByTagName("textarea");
      if (textareas.length > 0) {
        textareas[0].focus();
      }
    });
  });
  headerContainer.appendChild(header);
  headerContainer.appendChild(backButton);
  dom.appendChild(headerContainer);
  const headerBorder = crelt("div", {
    class: "cm-ext-assistant-header-border"
  });
  dom.appendChild(headerBorder);
  const description = crelt("div", { class: "cm-ext-assistant-settings-description" }, "Keys are stored locally and never transmitted");
  dom.appendChild(description);
  const inputsContainer = crelt("div", {
    class: "cm-ext-assistant-settings-inputs"
  });
  AVAILABLE_MODELS.forEach((model) => {
    const container = crelt("div", { class: "cm-ext-assistant-provider" });
    const label = crelt("div", { class: "cm-ext-assistant-provider-header" }, formatProviderName(model.provider));
    container.appendChild(label);
    const messageContainer = crelt("div", {
      class: "cm-ext-assistant-provider-message" + (state.apiKeys[model.provider] ? " has-api-key" : "")
    });
    if (state.apiKeys[model.provider]) {
      const checkmark = crelt("span", { class: "checkmark" }, "\u2713");
      messageContainer.appendChild(checkmark);
      messageContainer.appendChild(document.createTextNode("API key configured."));
      const resetButton = crelt("button", { class: "cm-ext-assistant-settings-button" }, "Reset key");
      resetButton.addEventListener("click", () => {
        input.value = "";
        view.dispatch({
          effects: setApiKeyEffect.of({
            provider: model.provider,
            key: ""
          })
        });
        messageContainer.classList.remove("has-api-key");
        inputGroup.classList.remove("has-api-key");
      });
      messageContainer.appendChild(resetButton);
    }
    const inputGroup = crelt("div", {
      class: "cm-ext-assistant-provider-input" + (state.apiKeys[model.provider] ? " has-api-key" : "")
    });
    const input = crelt("input", {
      type: "password",
      value: state.apiKeys[model.provider] || "",
      placeholder: model.provider === "google" ? "AIzaSy..." : model.provider === "openai" ? "sk-000000000000000000000000000000000000000000000000" : "ottggm..."
    });
    input.addEventListener("keydown", (e) => {
      if (e.key === "Enter") {
        input.blur();
      }
    });
    input.addEventListener("blur", () => {
      if (input.value) {
        view.dispatch({
          effects: setApiKeyEffect.of({
            provider: model.provider,
            key: input.value
          })
        });
      }
    });
    input.addEventListener("focus", () => {
      input.style.borderColor = "var(--cm-accent-color, #4a9eff)";
      input.style.boxShadow = "0 0 0 1px var(--cm-accent-color, #4a9eff)";
    });
    input.addEventListener("blur", () => {
      input.style.borderColor = "var(--cm-border-color, rgba(255, 255, 255, 0.1))";
      input.style.boxShadow = "none";
    });
    inputGroup.appendChild(input);
    container.appendChild(messageContainer);
    container.appendChild(inputGroup);
    inputsContainer.appendChild(container);
  });
  dom.appendChild(inputsContainer);
}
function renderMessage(message, container, scrollToBottom) {
  if (message.status === "streaming" && !message.content) {
    const loadingContainer = crelt("div", {
      class: "cm-ext-assistant-loading"
    });
    const spinner = crelt("div", { class: "cm-ext-assistant-spinner" });
    loadingContainer.appendChild(spinner);
    container.appendChild(loadingContainer);
  }
  if (message.content || message.status !== "streaming") {
    const messageEl = crelt("div", {
      class: `cm-ext-assistant-message ${message.role === "user" ? "cm-ext-assistant-message-user" : "cm-ext-assistant-message-bot"}`
    });
    const contentEl = crelt("div", {
      class: "cm-ext-assistant-message-content"
    });
    const codeBlocks = parseCodeBlocks(message.content);
    let lastPos = 0;
    const segments = [];
    codeBlocks.forEach((block) => {
      if (block.from > lastPos) {
        segments.push({
          type: "text",
          content: message.content.slice(lastPos, block.from)
        });
      }
      segments.push({
        type: "code",
        content: block.code,
        language: block.language
      });
      lastPos = block.to;
    });
    if (message.status === "streaming") {
      const remainingContent = message.content.slice(lastPos);
      const incompleteBlockMatch = /```([a-zA-Z]*)\n([\s\S]*)$/.exec(remainingContent);
      if (incompleteBlockMatch) {
        const incompleteBlockStart = remainingContent.lastIndexOf("```");
        if (incompleteBlockStart > 0) {
          segments.push({
            type: "text",
            content: remainingContent.slice(0, incompleteBlockStart)
          });
        }
        segments.push({
          type: "incomplete-code",
          content: "",
          language: incompleteBlockMatch[1] || null
        });
      } else {
        if (remainingContent) {
          segments.push({
            type: "text",
            content: remainingContent
          });
        }
      }
    } else {
      if (lastPos < message.content.length) {
        segments.push({
          type: "text",
          content: message.content.slice(lastPos)
        });
      }
    }
    segments.forEach((segment) => {
      if (segment.type === "code" || segment.type === "incomplete-code") {
        const codeBlockContainer = crelt("div", {
          class: "cm-ext-assistant-code"
        });
        const widget = new CodeBlockWidget(segment.type === "code" ? segment.content : "", segment.language || null, segment.type === "incomplete-code");
        codeBlockContainer.appendChild(widget.toDOM());
        contentEl.appendChild(codeBlockContainer);
      } else if (segment.type === "text") {
        const textContainer = crelt("div", {
          class: "cm-ext-assistant-text-container"
        });
        const inlineCodeRegex = /(?<!`)`([^`]+)`(?!`)/g;
        const processedText = segment.content.replace(inlineCodeRegex, (_, code) => `<code class="cm-ext-assistant-inline-code">${code}</code>`);
        textContainer.innerHTML = processedText;
        contentEl.appendChild(textContainer);
      }
    });
    messageEl.appendChild(contentEl);
    container.appendChild(messageEl);
    scrollToBottom();
  }
}

// src/sidebar/index.ts
import { ViewPlugin, EditorView as EditorView3 } from "@codemirror/view";
import {
  StateField as StateField3,
  StateEffect as StateEffect3,
  Facet
} from "@codemirror/state";
import crelt2 from "crelt";

// src/sidebar/styles.ts
var styles = {
  sidebar: "cm-ext-sidebar",
  resizeHandle: "cm-ext-sidebar-resize",
  panelContainer: "cm-ext-sidebar-panel"
};
var defaultSidebarOptions = {
  overlay: true
};

// src/themes/sidebar.ts
import { EditorView as EditorView2 } from "@codemirror/view";
var sidebarThemeVariables = {
  "--cm-ext-sidebar-bg": "#21222c",
  "--cm-ext-sidebar-border": "rgba(255, 255, 255, 0.2)",
  "--cm-ext-sidebar-resize-hover": "rgba(255, 255, 255, 0.1)",
  "--cm-ext-sidebar-resize-active": "rgba(255, 255, 255, 0.1)",
  "--cm-ext-sidebar-width": "250px",
  "--cm-ext-sidebar-resize-handle-width": "8px",
  "--cm-ext-sidebar-min-width": "150px",
  "--cm-ext-sidebar-max-width": "800px"
};
var sidebarTheme = {
  theme: EditorView2.theme({
    "&": {
      height: "100vh",
      display: "flex !important",
      flexDirection: "row !important",
      position: "relative",
      ...Object.entries(sidebarThemeVariables).reduce((acc, [key, value]) => ({
        ...acc,
        [key]: value
      }), {})
    },
    ".cm-ext-sidebar": {
      height: "100%",
      width: "var(--cm-ext-sidebar-width)",
      display: "flex",
      flexDirection: "column",
      flexShrink: "0",
      transition: "transform 0.2s ease-in-out",
      transform: "translateX(0)",
      backgroundColor: "var(--cm-ext-sidebar-bg)",
      position: "relative",
      '&[data-overlay="true"]': {
        zIndex: "10",
        position: "absolute",
        top: "0",
        height: "100%",
        '&[data-dock="left"]': {
          left: "0"
        },
        '&[data-dock="right"]': {
          right: "0"
        }
      },
      '&[data-overlay="false"]': {
        position: "relative",
        zIndex: "1",
        overflow: "hidden",
        flexShrink: "0",
        '&[data-dock="left"]': {
          order: -1
        },
        '&[data-dock="right"]': {
          order: 1
        }
      },
      '&[data-visible="false"]': {
        transform: "translateX(100%)",
        display: "none"
      }
    },
    ".cm-ext-sidebar-panel": {
      height: "100%",
      overflow: "auto",
      backgroundColor: "var(--cm-ext-sidebar-bg)",
      '&[data-dock="left"]': {
        borderRight: "0.5px solid var(--cm-ext-sidebar-border)"
      },
      '&[data-dock="right"]': {
        borderLeft: "0.5px solid var(--cm-ext-sidebar-border)"
      }
    },
    ".cm-ext-sidebar-resize": {
      position: "absolute",
      top: "0",
      width: "var(--cm-ext-sidebar-resize-handle-width)",
      height: "100%",
      cursor: "col-resize",
      zIndex: "20",
      backgroundColor: "transparent",
      transition: "none"
    },
    ".cm-ext-sidebar-resize:hover": {
      backgroundColor: "var(--cm-ext-sidebar-resize-hover)",
      transitionProperty: "background-color",
      transitionDuration: "0.2s",
      transitionTimingFunction: "ease",
      transitionDelay: "150ms"
    },
    ".cm-ext-sidebar-resize:active": {
      backgroundColor: "var(--cm-ext-sidebar-resize-active)",
      transition: "none"
    },
    "&.cm-ext-sidebar-dragging": {
      cursor: "col-resize",
      userSelect: "none"
    },
    ".cm-scroller": {
      flex: "1 1 auto !important",
      minWidth: "0 !important",
      position: "relative",
      overflow: "auto"
    }
  })
};

// src/sidebar/index.ts
var sidebarPanel = Facet.define({
  combine: (values) => values.flat()
});
var toggleSidebarEffect = StateEffect3.define();
var updateSidebarOptionsEffect = StateEffect3.define();
var setActivePanelEffect = StateEffect3.define();
var sidebarStates = new Map;
var createSidebarState = (id, initialOptions) => {
  return StateField3.define({
    create: () => ({
      visible: initialOptions.initiallyOpen ?? false,
      options: initialOptions,
      activePanelId: initialOptions.initiallyOpen && initialOptions.initialPanelId ? initialOptions.initialPanelId : null
    }),
    update(value, tr) {
      let newState = value;
      for (const e of tr.effects) {
        if (e.is(toggleSidebarEffect) && e.value.id === id) {
          newState = { ...newState, visible: e.value.visible };
        } else if (e.is(updateSidebarOptionsEffect) && e.value.id === id) {
          newState = {
            ...newState,
            options: { ...newState.options, ...e.value }
          };
        } else if (e.is(setActivePanelEffect) && e.value.id === id) {
          newState = {
            ...newState,
            activePanelId: e.value.panelId
          };
        }
      }
      return newState;
    }
  });
};
var toggleSidebarCommand = (view, sidebarId) => {
  const stateField = sidebarStates.get(sidebarId);
  if (!stateField) {
    debug3("No state field for sidebar:", sidebarId);
    return false;
  }
  const state = view.state.field(stateField);
  const newVisible = !state.visible;
  debug3("Toggling sidebar:", sidebarId, "from:", state.visible, "to:", newVisible);
  const effects = [
    toggleSidebarEffect.of({
      id: sidebarId,
      visible: newVisible
    })
  ];
  if (newVisible && !state.activePanelId) {
    const panels = view.state.facet(sidebarPanel);
    const panel = panels.find((p) => p.id === sidebarId);
    if (panel) {
      effects.push(setActivePanelEffect.of({
        id: sidebarId,
        panelId: panel.id
      }));
    }
  }
  view.dispatch({ effects });
  return true;
};
var debug3 = (...args) => logger_default.debug({ module: "Sidebar" }, args.join(" "));
var createSidebarPlugin = (id) => ViewPlugin.fromClass(class {
  dom;
  panelContainer;
  activePanel = null;
  resizeHandle;
  initialWidth = 0;
  initialX = 0;
  isDragging = false;
  constructor(view) {
    this.dom = crelt2("div", {
      class: styles.sidebar,
      "data-sidebar-id": id
    });
    this.panelContainer = crelt2("div", {
      class: styles.panelContainer
    });
    this.resizeHandle = crelt2("div", {
      class: styles.resizeHandle
    });
    this.dom.appendChild(this.resizeHandle);
    this.dom.appendChild(this.panelContainer);
    const stateField = sidebarStates.get(id);
    const state = view.state.field(stateField);
    this.applySidebarStyles(state.options);
    this.updateVisibility(state.visible);
    view.dom.appendChild(this.dom);
    view.dom.style.position = "relative";
    this.renderActivePanel(view, state);
    this.setupResizeHandle();
  }
  update(update) {
    const stateField = sidebarStates.get(id);
    const state = update.state.field(stateField);
    const oldState = update.startState.field(stateField);
    if (state.visible !== oldState.visible) {
      debug3("Visibility changed:", id, state.visible);
      this.applySidebarStyles(state.options);
      this.updateVisibility(state.visible);
      if (!state.visible) {
        update.view.focus();
      }
      const panels = update.state.facet(sidebarPanel);
      const activePanel = panels.find((p) => p.id === state.activePanelId);
      if (activePanel?.onVisibilityChange) {
        activePanel.onVisibilityChange(update.view, state.visible);
      }
    }
    if (state.options !== oldState.options) {
      debug3("Options changed:", id, state.options);
      this.applySidebarStyles(state.options);
    }
    if (state.activePanelId !== oldState.activePanelId) {
      debug3("Active panel changed:", id, state.activePanelId);
      this.renderActivePanel(update.view, state);
    }
  }
  destroy() {
    debug3("Destroying sidebar plugin:", id);
    this.dom.remove();
  }
  updateVisibility(visible) {
    this.dom.setAttribute("data-visible", visible.toString());
  }
  setupResizeHandle() {
    const startDragging = (e) => {
      e.preventDefault();
      this.isDragging = true;
      this.initialX = e.clientX;
      this.initialWidth = this.dom.offsetWidth;
      document.addEventListener("mousemove", onDrag);
      document.addEventListener("mouseup", stopDragging);
      document.body.classList.add("cm-ext-sidebar-dragging");
    };
    const onDrag = (e) => {
      if (!this.isDragging)
        return;
      const stateField = sidebarStates.get(id);
      const editorElement = this.dom.closest(".cm-editor");
      if (!editorElement)
        return;
      const view = EditorView3.findFromDOM(editorElement);
      if (!view)
        return;
      const state = view.state.field(stateField);
      const delta = e.clientX - this.initialX;
      let newWidth = state.options.dock === "left" ? this.initialWidth + delta : this.initialWidth - delta;
      const minWidth = parseInt(sidebarThemeVariables["--cm-ext-sidebar-min-width"]);
      const maxWidth = parseInt(sidebarThemeVariables["--cm-ext-sidebar-max-width"]);
      newWidth = Math.max(minWidth, Math.min(maxWidth, newWidth));
      this.dom.style.setProperty("--cm-ext-sidebar-width", `${newWidth}px`);
    };
    const stopDragging = () => {
      if (!this.isDragging)
        return;
      this.isDragging = false;
      document.removeEventListener("mousemove", onDrag);
      document.removeEventListener("mouseup", stopDragging);
      document.body.classList.remove("cm-ext-sidebar-dragging");
      const stateField = sidebarStates.get(id);
      const editorElement = this.dom.closest(".cm-editor");
      if (!editorElement)
        return;
      const view = EditorView3.findFromDOM(editorElement);
      if (!view)
        return;
      view.dispatch({
        effects: updateSidebarOptionsEffect.of({
          ...view.state.field(stateField).options,
          width: this.dom.style.getPropertyValue("--cm-ext-sidebar-width")
        })
      });
    };
    this.resizeHandle.addEventListener("mousedown", startDragging);
  }
  applySidebarStyles(options) {
    const {
      width,
      backgroundColor,
      dock = "left",
      overlay = true
    } = options;
    this.dom.setAttribute("data-dock", dock);
    this.dom.setAttribute("data-overlay", overlay.toString());
    this.panelContainer.setAttribute("data-dock", dock);
    if (width) {
      this.dom.style.setProperty("--cm-ext-sidebar-width", width);
    }
    if (backgroundColor) {
      this.dom.style.setProperty("--cm-ext-sidebar-bg", backgroundColor);
    }
    if (dock === "left") {
      this.resizeHandle.style.left = "auto";
      this.resizeHandle.style.right = "-2px";
    } else {
      this.resizeHandle.style.left = "-2px";
      this.resizeHandle.style.right = "auto";
    }
    const editor = this.dom.closest(".cm-editor");
    if (editor) {
      editor.style.position = "relative";
    }
  }
  renderActivePanel(view, state) {
    if (this.activePanel) {
      this.activePanel.remove();
      this.activePanel = null;
    }
    if (state.activePanelId) {
      const panels = view.state.facet(sidebarPanel);
      const panel = panels.find((p) => p.id === state.activePanelId);
      if (panel) {
        this.activePanel = panel.create(view);
        this.panelContainer.appendChild(this.activePanel);
      }
    }
  }
});
function createSidebar(options) {
  const id = options.id;
  const stateField = createSidebarState(id, {
    ...defaultSidebarOptions,
    ...options
  });
  sidebarStates.set(id, stateField);
  return [stateField, createSidebarPlugin(id), sidebarTheme.theme];
}

// src/sidebar/keymap.ts
import { keymap } from "@codemirror/view";
var debug4 = (...args) => logger_default.debug("[Sidebar Keymap]", ...args);
function createSidebarBindings(sidebarId, key) {
  const bindings = [];
  debug4("Creating keymap bindings for sidebar:", sidebarId, key);
  if (typeof key === "string") {
    debug4("Adding universal key binding:", key);
    bindings.push({
      key,
      run: (view) => {
        debug4("Triggered universal key binding:", key);
        return toggleSidebarCommand(view, sidebarId);
      }
    });
  } else {
    if (key.mac) {
      debug4("Adding Mac key binding:", key.mac);
      bindings.push({
        mac: key.mac,
        run: (view) => {
          debug4("Triggered Mac key binding:", key.mac);
          return toggleSidebarCommand(view, sidebarId);
        }
      });
    }
    if (key.win) {
      debug4("Adding Windows key binding:", key.win);
      bindings.push({
        win: key.win,
        run: (view) => {
          debug4("Triggered Windows key binding:", key.win);
          return toggleSidebarCommand(view, sidebarId);
        }
      });
    }
  }
  return bindings;
}
function createSidebarKeymap(sidebarId, keymapConfig) {
  if (!keymapConfig) {
    debug4("No keymap config provided for sidebar:", sidebarId);
    return [];
  }
  debug4("Creating keymap extension for sidebar:", sidebarId, keymapConfig);
  return keymap.of(createSidebarBindings(sidebarId, keymapConfig));
}

// src/assistant/index.ts
import { Facet as Facet2 } from "@codemirror/state";
var debug5 = (...args) => logger_default.debug("[Assistant]", ...args);
function setupPanelKeyboardShortcuts(dom, view, keyConfig) {
  const isMac = /Mac/.test(navigator.platform);
  const keys = typeof keyConfig === "string" ? { mac: keyConfig, win: keyConfig } : keyConfig;
  const targetKey = (isMac ? keys.mac : keys.win)?.toLowerCase();
  if (!targetKey)
    return;
  debug5("Setting up panel keyboard shortcuts:", { isMac, targetKey });
  dom.addEventListener("keydown", (event) => {
    const modKey = isMac ? event.metaKey : event.ctrlKey;
    if (!modKey)
      return;
    const pressedKey = `${isMac ? "cmd" : "ctrl"}-${event.key.toLowerCase()}`;
    debug5("Key pressed in panel:", pressedKey);
    if (pressedKey === targetKey) {
      event.preventDefault();
      event.stopPropagation();
      debug5("Executing panel keyboard shortcut:", targetKey);
      toggleSidebarCommand(view, "ai-assistant");
    }
  });
}
var assistantPanelSpec = {
  id: "ai-assistant",
  create(view) {
    const dom = document.createElement("div");
    dom.className = "cm-assistant-content";
    renderAssistantPanel(dom, view);
    const keymapOpt = view.state.facet(assistantKeymap);
    if (keymapOpt) {
      setupPanelKeyboardShortcuts(dom, view, keymapOpt);
    }
    return dom;
  },
  update(view) {
    const dom = view.dom.querySelector(".cm-assistant-content");
    if (dom) {
      renderAssistantPanel(dom, view);
    }
  },
  onVisibilityChange(view, visible) {
    const root = view.dom;
    if (visible) {
      const panel = view.dom.querySelector('.cm-sidebar-panel-container[data-dock="right"]');
      const width = panel ? panel.style.width || "400px" : "400px";
      root.style.setProperty("--assistant-width", width);
      requestIdleCallback(() => {
        const textareas = view.dom.getElementsByTagName("textarea");
        if (textareas.length > 0) {
          textareas[0].focus();
        }
      });
    } else {
      root.style.setProperty("--assistant-width", "0px");
    }
  }
};
var assistantKeymap = Facet2.define({
  combine: (values) => values[0]
});
var assistantPlugin = ViewPlugin2.fromClass(class {
  update(update) {
    if (update.state.field(assistantState) !== update.startState.field(assistantState)) {
      const dom = update.view.dom.querySelector(".cm-ext-assistant-container, .cm-ext-assistant-settings");
      if (dom) {
        renderAssistantPanel(dom, update.view);
      }
    }
  }
});
function assistant(options = {}) {
  const {
    keymap: keymapOpt,
    initiallyOpen = false,
    ...sidebarOptions
  } = options;
  return [
    ...createSidebar({
      ...sidebarOptions,
      id: "ai-assistant",
      dock: "right",
      overlay: true,
      initiallyOpen,
      initialPanelId: "ai-assistant"
    }),
    assistantState,
    sidebarPanel.of(assistantPanelSpec),
    assistantPlugin,
    keymapOpt ? assistantKeymap.of(keymapOpt) : [],
    createSidebarKeymap("ai-assistant", keymapOpt)
  ];
}
export {
  updateMessageStatusEffect,
  toggleSettingsEffect,
  switchTabEffect,
  setApiKeyEffect,
  selectModelEffect,
  assistant,
  addMessageEffect
};
